// ENSAM 2005
// Workshop Infographie du 14/11 au 18/11
// Maison zHome
// macros generales

/////////////////////////////////////////////
//	MACROS :
/////////////////////////////////////////////

//	TEXTURES

#macro ma_texture( ccc, rrr )
	texture
	{	pigment	{ color rgbt ccc }
		finish
		{	ambient 	0.3
			diffuse		0.7
			specular	0.7
			roughness	0.01
			reflection	rrr
		}
	} 
#end

#macro texture_echiquier()
	texture
	{	pigment 
		{	checker
			color rgbt <1/4,1/4,1/4,0.8>
			color rgbt <3/4,3/4,3/4,0.0>
			scale <1, 1/64, 1>
			rotate <0,0,0>
			translate <1/2,0,1/2>
			
		}
		finish
		{	ambient 0.2
			diffuse 0.7
			specular 0.7
			roughness 0.01
			//reflection 0.8
		}
	}
	// interior { ior 1.3 }
#end

#declare BOIS 	= 0;
#declare MARBRE = 1;
#declare GRANIT = 2;

#macro textures_diverses( choix )
	texture
	{	pigment 
		{	#if (choix = BOIS) 		wood 	#end
			#if (choix = MARBRE) 	marble 	#end
			#if (choix = GRANIT) 	granite #end
			color_map
			{
				[ 0.1 color rgbt <1,0,0> ]
				[ 0.5 color rgb <0,0,0> ]
				[ 0.9 color rgbt <0,1,0> ]
			}
			turbulence 0.1
			rotate <0,0,30>
			scale 1/8
		}
		finish
		{	ambient 0.2
			diffuse 0.7
			specular 0.7
			roughness 0.01
			//reflection 0.8
		}
	}
	// interior { ior 1.3 }
#end

#macro texture_bump()
	normal
	{	
		bumps	1
		//waves 1/2
		scale 1/64
	}
#end

#declare GIF 	= 0;
#declare JPG	= 1;
#declare PNG	= 2;

#macro texture_image( nom, type_image )
	texture
	{	pigment
    	{	image_map
      		{
      			#if (type_image = GIF)	gif 	#end
      			#if (type_image = JPG)	jpeg 	#end
      			#if (type_image = PNG)	png 	#end
      			nom				
        		interpolate 2 		// lissage
        		once   				// pas de repetition
       		}
      		translate <-0.5,-0.5,0>	// centrage
     		scale 1/2				// taille
  		}
 		finish
		{	ambient 0.2
			diffuse 0.7
			specular 0.7
			roughness 0.01
		}
	}
#end

#macro geri()
object 
{	polygon { 5, <0,0>, < 1,0>, < 1, 1>, <0, 1>, <0,0> }
	texture
	{	pigment
		{	image_map 
			{ 	gif "geri.gif" 
				once 
				filter 0, 1.0 	// la couleur indice 0 est transparente
			}		
		}
		finish { ambient 0.2 diffuse 0.6 }
	}
	scale <1,2,1>
}
#end

#macro texte( chaine )		// texte( "Architecte" )
text 
{		ttf 
		"LucidaBrightRegular.ttf"
		chaine 
		1/4, 
		<0,0,0>
		ma_texture( <1,0,0>, 0.5 )
		scale <1,-1,1>
		rotate <-90,0,0>		
}
#end

//	GEOMETRIE

#macro grand_tore( ccc )
	object
	{	torus {  0.5, 0.2  }
		ma_texture(  ccc, 0.0 ) 
	}
#end

#macro petit_tore( aaa, ccc )
	object
	{	torus {  0.2, 0.05  }
		translate <0.5,0,0>
		rotate <90,aaa,0>
		ma_texture( ccc, 0 ) 
	}
#end

#macro multi_tores( N)
	union
	{
		#local i = 0;
			#while ( i<N )	
			petit_tore( 360/N*i, < 	1-i/(N-1), 	// rouge de 1  0
									i/(N-1), 	// vert de 0  1
									0,  		// bleu 0
									0> 			// transparent 0
									)
			#local i=i+1;
		#end
	}
#end

#macro anneau_1()
	difference
	{	multi_tores( 12 )
		grand_tore( <0,0,0> )
	}
#end

#macro anneau_2()
	difference
	{	grand_tore( <0,0,0> )
		multi_tores( 12 )
		translate <0.5,0,0>
		rotate <90,0,0>
	}
#end

#macro une_geometrie()
	difference
	{	intersection
		{	box { <-1/2,-1/2,-1/2>, <1/2,1/2,1/2> scale 6/8 }
			sphere { <0,0,0>  1/2 }
		}
		cylinder { <0,-1/2-1,0>, <0,1/2+1,0>, 1/4 }
		cylinder { <-1/2-1,0,0>, <1/2+1,0,0>, 1/4 }
	}
#end

#macro objet_test()
	object
	{	une_geometrie()
		ma_texture( <1,0,0>, 0 )
		#if ( RENDU_FINAL = true )
			texture_echiquier()
			textures_diverses( GRANIT )
			texture_bump()
		#end
		texture_image( "artist2_on.gif", GIF )
	}
#end

#macro tableau( nom_image, type_image, taille, rotation, translation )
	object
	{	box { -1, 1 scale <1/4, 1/4, 0.01> }
		pigment { color rgb <1,1,1> }
		texture_image( nom_image, type_image )
		scale taille
		rotate rotation
		translate translation
	}
#end

#macro cadre( e, h )
	prism 
	{	linear_sweep
  		0, h,
  		10,          	// nombre de points
  		// un premier contour
  		<-1,-1>, < 1,-1>, 
  		< 1, 1>, <-1, 1>, 
  		<-1,-1> 
		// le meme en plus petit :
  		<-1*e,-1*e>, < 1*e,-1*e>, 
  		< 1*e, 1*e>, <-1*e, 1*e>, 
  		<-1*e,-1*e>
    	//open
	scale 1/2
	}
#end

#macro ressort( tt )				//tt dans [0,1]
	#local sommets = array[8]
	{	<-1,-1,-1>, < 1,-1,-1>, < 1, 1,-1>, < 1, 1, 1>, 
		< 1,-1, 1>, <-1,-1, 1>, <-1,1,1>, <-1,1,-1> }
	#local points1 = points_spline( sommets, 1, true, 40 )
	#local points2 = points_spline( sommets, 2, true, 30 )
	#local points3 = points_spline( sommets, 2, true, 30 )
	union
	{	object { tracer_courbe( points1, 0.01, true ) ma_texture( <1,0,0>, 0 ) scale 1/2 }
		object { tracer_courbe( points2, 0.05, true ) ma_texture( <0,1,0>, 0.5 ) scale 1/2 }
		object { tracer_courbe( points3, 0.2, false ) ma_texture( <0,0,0>, 0.9 ) scale 1/4 }
		rotate <0, 360*tt, 0>
	}
#end
//	VISUALISATION

#macro axes()
	union
	{	cylinder { <-100,0,0>, <100,0,0>, 0.05 pigment { color rgb <1,0,0> } }
		cylinder { <0,-100,0>, <0,100,0>, 0.05 pigment { color rgb <0,1,0> } }
		cylinder { <0,0,-100>, <0,0,100>, 0.05 pigment { color rgb <0,0,1> } }
	}
#end

#macro sol( taille, translation, gris)
	object
	{	box { <-taille,-0.01,-taille>, <taille,0.01,taille> }
		ma_texture( <gris,gris,gris>, 0 )
		translate <0,translation, 0>
	}
#end

#declare OX 	= 1;
#declare OY 	= 2;
#declare OZ 	= 3;
#declare AXO_1 	= 4;
#declare AXO_2 	= 5;
#declare AXO_3 	= 6;
#declare AXO_4 	= 7;

#macro axono( choix, taille, cible )
	#switch (choix)
		#case (OX)		camera { orthographic location <-1,0,0> right <RATIO,0,0> look_at <0, 0, 0> scale 1/taille translate cible }	#break
		#case (OY)		camera { orthographic location <0,1,0> right <RATIO,0,0> look_at <0, 0, 0> scale 1/taille translate cible }		#break
		#case (OZ)		camera { orthographic location <0,0,1> right <RATIO,0,0> look_at <0, 0, 0> scale 1/taille translate cible }	#break
		#case (AXO_1)	camera { orthographic location <-1,1,-1> right <RATIO,0,0> look_at <0, 0, 0> scale 1/taille translate cible }	#break
		#case (AXO_2)	camera { orthographic location < 1,1,-1> right <RATIO,0,0> look_at <0, 0, 0> scale 1/taille translate cible }	#break
		#case (AXO_3)	camera { orthographic location < 1,1, 1> right <RATIO,0,0> look_at <0, 0, 0> scale 1/taille translate cible }	#break
		#case (AXO_4)	camera { orthographic location <-1,1, 1> right <RATIO,0,0> look_at <0, 0, 0> scale 1/taille translate cible }	#break
	#end
#end

#macro pers( oeil, cible )
	camera { location oeil right <RATIO,0,0> look_at cible }
#end


#macro eclairage_standard( nombre, dist )
	#switch (nombre)
		#case (1) 
			light_source { <-1,1,-1>*dist color rgb <1,1,1>	}	#break
		#case (2) 
			light_source { <-1,1,-1>*dist color rgb <1,1,1> }	
			light_source { < 1,1,-1>*dist color rgb <1,1,1> }	#break
		#case (3) 
			light_source { <-1,1,-1>*dist color rgb <1,1,1> }	
			light_source { < 1,1,-1>*dist color rgb <1,1,1> }
			light_source { < 0,2, 2>*dist color rgb <1,1,1> }	#break
	#end
#end

#macro fond( gris )
	background { color rgb gris }
#end

#macro ciel()
	sky_sphere
	{	pigment
  		{	granite	// gradient y
    		color_map { [0.0 color rgb <0.7,0.7,1.0>] [1.0 color blue 0.5] }
    		scale 1/1
    		turbulence 0.1
  		}
	}
#end

#macro spot( pos, cible, rayon, taille_boule, couleur )
	light_source
	{	<0,0,0>				// par defaut
		color rgb couleur
		spotlight			
		point_at cible
  		radius rayon		// rayon interne			
		tightness 1			// passage lumiere -> ombre
		falloff rayon+10	// rayon externe
		looks_like 
		{	sphere { <0,0,0>, taille_boule }
			texture
			{	pigment { color rgb couleur }
				finish { ambient 1 } 	// boule blanche
			} 
		}
		translate pos	// ici on gere la position
	}
#end

#macro les_4_spots()
	union
	{	spot( <-1/2,0.9,-1/2>, <0,-1,0>, 30, 0.05, <1,1,0> )
		spot( < 1/2,0.9,-1/2>, <0,-1,0>, 30, 0.05, <0,1,1> )
		spot( < 1/2,0.9, 1/2>, <0,-1,0>, 30, 0.05, <1,0,1> )
		spot( <-1/2,0.9, 1/2>, <0,-1,0>, 30, 0.05, <1,1,1> )
	}
#end

//	UN PEU DE MATHS :

#macro segment ( p0, p1, tt )
	(1-tt)*p0 + tt*p1;
#end

#macro cercle( rayon, tt, translation )
	#local xx = rayon * cos( 2*pi*tt );
	#local zz = rayon * sin( 2*pi*tt );
	centre + <xx,0,zz>;
#end

#macro helice( _r0, _r1, tt, h0, h1, N )
	#if (tt<0.5)	#local rr = (1-tt)*_r0 + tt*_r1;	#end
	#if (tt>=0.5)	#local rr = (1-tt)*_r1 + tt*_r0;	#end
	#local xx = rr * cos( 2*N*pi*tt );
	#local zz = rr * sin( 2*N*pi*tt );
	#local yy = (1-tt)*h0 + tt*h1;
	<xx,yy,zz>;
#end

#macro courbe( N )
	#local i=0; #while (i<N)
		#local p = helice(  0.2, 2.0, i/N, -2.0, 2.0, 4 )
		sphere { p, 0.02 pigment { color rgb <0,1,0> } }
	#local i=i+1; #end
#end

/////////////////////////////////////////////////////////////////////////
/*
	SPLINES:
	polylignes ou polycourbes issues de la concatenation de 
	segments, de quadriques, de cubiques  ;
*/
/////////////////////////////////////////////////////////////////////////

#macro lin(a, p0, p1)		// p0 et p1 sont deux points de l'espace,
	(1-a)*p0 + 				// a est suppose varier dans l'intervalle [0,1]
	    a*p1				// le point calcule se deplacera de p0 a p1
#end

#macro quad(a, p0, p1, p2)
	(1-a)*(1-a)*p0 + 		// noter la forme: 1*k0*p0 + 2*k1*p1 + 1*k2*p2
	  2*(1-a)*a*p1 + 		//							1 2 1
	        a*a*p2
#end

#macro cub(a, p0, p1, p2, p3)
	(1-a)*(1-a)*(1-a)*p0 + 	// noter la forme: 1*k0*p0 + 3*k1*p1 + 3*k2*p2 + 1*k3*p3
	  3*(1-a)*(1-a)*a*p1 + 	// 							1 3 3 1
	      3*(1-a)*a*a*p2 + 	// forme des coefficients du triangle de Pascal, au dela
	            a*a*a*p3	// on aurait: 1 4 6 4 1, puis 1 5 10 10 5 1, etc...
#end


#macro points_spline( pp, degre, extremites, nombre )
	#local N = dimension_size( pp, 1 );
	#local NN = nombre*(N-degre);
	#local tab = array[NN]
	#local k = 0;
	#local j = 0; #while (j<N-degre)
	#if (!extremites)	// pour accrocher la courbe aux extremites
		#switch (degre)
		#case (1)
			#local q0 = pp[j];
			#local q1 = pp[j+1];
		#break
		#case (2)
			#local q0 = (pp[j]+pp[j+1])/2;
			#local q1 = pp[j+1];
			#local q2 = (pp[j+1]+pp[j+2])/2;
		#break
		#case (3)
			#local q0 = (pp[j] + 4*pp[j+1] +   pp[j+2] )/6;
			#local q1 = ( 4*pp[j+1] + 2*pp[j+2] )/6;
			#local q2 = ( 2*pp[j+1] + 4*pp[j+2] )/6;
			#local q3 = ( pp[j+1] + 4*pp[j+2] + pp[j+3] )/6;
		#break
		#end
	#else
		#switch (degre)
		#case (1)
			#local q0 = pp[j];
			#local q1 = pp[j+1];
		#break
		#case (2)
			#if (j!=0)		#local q0 = (pp[j]+pp[j+1])/2;
			#else			#local q0 = pp[j];		#end
							#local q1 = pp[j+1];
			#if (j!=N-2-1)	#local q2 = (pp[j+1]+pp[j+2])/2;
			#else			#local q2 = pp[j+2];	#end
		#break
		#case (3)
			#if (j!=0)		#local q0 = (pp[j] + 4*pp[j+1] +   pp[j+2] )/6;
			#else			#local q0 = pp[j];		#end
							#local q1 = ( 4*pp[j+1] + 2*pp[j+2] )/6;
							#local q2 = ( 2*pp[j+1] + 4*pp[j+2] )/6;
			#if (j!=N-3-1)	#local q3 = ( pp[j+1] + 4*pp[j+2] + pp[j+3] )/6;
			#else			#local q3 = pp[j+3];	#end
		#break
		#end
	#end
		// pour eviter le redoublement au droit des noeuds
	//	#if (j=N-degre-1)	#local fin = 0; #else 	#local fin = 1; #end
		#local fin = -1;
		#local i=0; #while (i<nombre)
			#switch (degre)
				#case (1) #local tab[k] = lin(i/(nombre-1-fin), q0, q1);			#break
				#case (2) #local tab[k] = quad(i/(nombre-1-fin), q0, q1, q2);		#break
				#case (3) #local tab[k] = cub(i/(nombre-1-fin), q0, q1, q2, q3);	#break
			#end
			#local k=k+1;
		#local i=i+1; #end
	#local j=j+1; #end
	tab
#end

#macro tracer_courbe( pp, taille, lisse )
	#local N = dimension_size(pp,1);
	#if (!lisse)
	union
	{	#local i=0; #while (i<N)
			sphere 		{ pp[i], taille }
		#local i=i+1; #end
	}
	#else
	union
	{	#local i=0; #while (i<N-1)
			cylinder 	{ pp[i], pp[i+1], taille }
		#local i=i+1; #end
	}
	#end
#end

/////////////////////////////////////////////////////////////////////////
